<?php

/**
 * Usage:
 * - Copy icao24plus.txt into same folder as script
 * - Create subfolder db if not exists
 * - On command line run: php index.php
 * 
 * When done:
 * - delete all files in dump1090-fa/mutability web share in
 *   /usr/share/dump1090-fa/html/db
 * - DO NOT delete aircraft_types!!!
 * - Copy content of db folder to dump1090-fa/mutability web
 *   share in /usr/share/dump1090-fa/html/db
 * 
 * The dump-1090 map shows all aircrafts from database with a
 * leading # on registration.
 * 
 * 2016 www.mictronics.de
 * 
 */

$input_file = "icao24plus.txt";
        
function read_icao24plus($infile, &$blocks)
{
    echo "Reading from: $infile\n";
    $ac_count = 0;
    
    if(file_exists($infile) == FALSE) die("Input file not found!");
    $file = fopen($infile, "r");
    
    fgets($file); // Skip first line with creation date and time
    
    while(!feof($file))
    {
        $row = fgetcsv($file, 0, "\t");
        $icao24 = $row[0];
        
        if(count($row) < 3) continue;
        
        ++$ac_count;
        $icao24 = strtoupper($icao24);
        $bkey = "#".substr($icao24, 0, 1);
        $dkey = "#".substr($icao24, 1);
        
        if(count($row) < 4)
        {
            $entry = array("r" => "# ".$row[1], "t" => $row[2], "desc" => "");
        }
        else
        {
            $entry = array("r" => "# ".$row[1], "t" => $row[2], "desc" => $row[3]);
        }
        $blocks[$bkey][$dkey] = $entry;
    }
    
    echo "Read $ac_count aircrafts from $infile\n";
}

function writedb(&$blocks, $todir, $blocklimit, $debug)
{
    $block_count = 0;
    echo "Writing blocks:\n";
    
    if(asort($blocks) == FALSE) die("Error sorting blocks!");
  
    $queue = array_keys($blocks);
    
    while(count($queue) > 0)
    {
        $bkey = array_shift($queue);
        if($bkey === NULL) break;
        
        $blockdata = $blocks[$bkey];

        if(count($blockdata) > $blocklimit)
        {
            if($debug) echo "Spliting block ".substr($bkey,1)." with ".count($blockdata)." entries... ";
            $children = array();
            foreach(array_keys($blockdata) as $dkey)
            {
                $new_bkey = "#".substr($bkey,1).$dkey[1];
                $new_dkey = "#".substr($dkey, 2);
                $children[$new_bkey][$new_dkey] = $blockdata[$dkey];
            }
            
            if(asort($children) == FALSE) die("Error sorting childrens!");
            $blockdata = array();
            $retained = 1;
            $c_bkeys = array_keys($children);
          
            while(count($children[$c_bkeys[0]]) + $retained < $blocklimit)
            {
                foreach($children[$c_bkeys[0]] as $c_dkey => $var)
                {
                    $s1 = substr($c_bkeys[0],-1);
                    $s2 = substr($c_dkey,1);
                    $blockdata[$s1.$s2] = $var;
                    ++$retained;
                }
                array_shift($c_bkeys);            
            }
            
            if($debug) echo count($c_bkeys)." children created $retained retainted in parent\n";
            
            $blockdata["children"] = $c_bkeys;
            $blocks[$bkey] = $blockdata;
            
            foreach(array_keys($children) as $key)
            {
                $blocks[$key] = $children[$key];
            }
            $queue = array_merge($queue, $c_bkeys);
        }
        
        $path = $todir.substr($bkey,1).".json";
      
        if($debug) echo "Writing ".count($blockdata)." entries to $path\n";
        else echo substr($bkey,1)." ";
        $dbfile = fopen($path,"w");
        /* Generate JSON output */

        $items = Array();
        foreach($blocks[$bkey] as $key => $value)
        {
            // Ensure all strings are UTF-8
            array_walk_recursive($value, 'encode_items');
            // Remove # from key during JSON conversion
            if($key == "children")
            {
                array_walk($value, 'trimkey');
            }
            $items[] = json_encode(ltrim($key,'#')) . ':' . json_encode($value);
        }        

        $json = '{' . implode(',', $items) . '}';
        fputs($dbfile, $json);
        
        fclose($dbfile);
        ++$block_count;
    }
    
    echo "done.\nWrote $block_count blocks.\n";
}

function encode_items(&$value, $key)
{
    $value = utf8_encode($value);
}

function trimkey(&$value, $key)
{
    $value = ltrim($value, "#");
}

function validate_json($icao24, $level) {
    
    $bkey = substr($icao24,0,$level);
    $dkey = substr($icao24, $level);
    
    $json = file_get_contents('db/'.$bkey.'.json');
    $jsonIterator = new RecursiveIteratorIterator(
    new RecursiveArrayIterator(json_decode($json, TRUE)),
    RecursiveIteratorIterator::SELF_FIRST);
    
    foreach ($jsonIterator as $key => $val) {
        if(is_array($val) && ($dkey==$key)) {
            //echo "$bkey\n";
            return true;
        }
        
        if(is_array($val) && ($key=='children')) {
            $children = $val;
            
            $subkey = $bkey.substr($dkey,0,1);
            if(in_array($subkey, $children)){
                return validate_json($icao24, $level+1);
            }
        }
    }
}

$blocks = array();
/* Create JSON database */
read_icao24plus($input_file, $blocks);
writedb($blocks, "db\\", 2500, true);

/* Validate JSON database against input */
/*
if(file_exists($input_file) == FALSE) die("Input file not found!");
$file = fopen($input_file, "r");

fgets($file); // Skip first line with creation date and time

while(!feof($file))
{
    $row = fgetcsv($file, 0, "\t");
    $icao24 = $row[0];

    if(count($row) < 3) continue;
    $icao24 = strtoupper($icao24);
    if(validate_json($icao24, 1) === false){
        echo "$icao24 not found\n";
    }
}
*/